@@ -3,8 +3,8 @@ package input
 
 import (
 	"os"
+	"path/filepath"
 	"strings"
-	"sync"
 
 	"github.com/atinylittleshell/gsh/internal/repl/render"
 	"github.com/charmbracelet/lipgloss"
@@ -54,11 +54,10 @@ type Highlighter struct {
 	// modified in .gshenv or .gsh_profile) instead of the OS process's PATH.
 	getEnv func(name string) string
 
-	// Command existence cache (for PATH lookups)
-	// The cache key includes the PATH value to handle PATH changes.
-	cmdCacheMu sync.RWMutex
-	cmdCache   map[string]bool
-	cachedPath string // The PATH value when the cache was populated
+	// getWorkingDir, if set, is used to resolve relative paths. This keeps
+	// highlighting aligned with the shell's current working directory, which may
+	// differ from the process working directory when users run `cd` inside gsh.
+	getWorkingDir func() string
 
 	// Styles for different token types
 	styles map[TokenType]lipgloss.Style
@@ -71,13 +70,20 @@ type Highlighter struct {
 //
 // If getEnv is non-nil, it is used to get environment variables from the shell,
 // allowing the highlighter to use the shell's PATH and variable values.
-func NewHighlighter(aliasExists func(name string) bool, getEnv func(name string) string) *Highlighter {
+//
+// If getWorkingDir is non-nil, it is used to resolve relative command paths so
+// highlighting matches the shell's current working directory.
+func NewHighlighter(
+	aliasExists func(name string) bool,
+	getEnv func(name string) string,
+	getWorkingDir func() string,
+) *Highlighter {
 	h := &Highlighter{
-		parser:      syntax.NewParser(),
-		aliasExists: aliasExists,
-		getEnv:      getEnv,
-		cmdCache:    make(map[string]bool),
-		styles:      make(map[TokenType]lipgloss.Style),
+		parser:        syntax.NewParser(),
+		aliasExists:   aliasExists,
+		getEnv:        getEnv,
+		getWorkingDir: getWorkingDir,
+		styles:        make(map[TokenType]lipgloss.Style),
 	}
 
 	// Initialize styles
@@ -97,14 +103,13 @@ func NewHighlighter(aliasExists func(name string) bool, getEnv func(name string)
 }
 
 // commandExists checks if a command exists in PATH, is an alias, or is a built-in.
-// PATH lookups are cached; alias lookups are not.
 func (h *Highlighter) commandExists(cmd string) bool {
-	// Check built-ins first (no caching needed, it's a map lookup)
+	// Check built-ins first
 	if builtinCommands[cmd] {
 		return true
 	}
 
-	// Treat aliases as valid commands (aliases can change at runtime).
+	// Treat aliases as valid commands
 	if h.aliasExists != nil && h.aliasExists(cmd) {
 		return true
 	}
@@ -117,42 +122,22 @@ func (h *Highlighter) commandExists(cmd string) bool {
 		}
 	}
 
-	// Check cache for external commands
-	// Invalidate cache if PATH has changed
-	h.cmdCacheMu.RLock()
-	exists, cached := h.cmdCache[cmd]
-	pathChanged := h.cachedPath != currentPath
-	h.cmdCacheMu.RUnlock()
-
-	if cached && !pathChanged {
-		return exists
-	}
-
-	// If PATH changed, clear the entire cache
-	if pathChanged {
-		h.cmdCacheMu.Lock()
-		h.cmdCache = make(map[string]bool)
-		h.cachedPath = currentPath
-		h.cmdCacheMu.Unlock()
-	}
-
 	// Check if command exists in the shell's PATH
-	exists = h.lookupCommandInPath(cmd, currentPath)
-
-	// Cache the result
-	h.cmdCacheMu.Lock()
-	h.cmdCache[cmd] = exists
-	h.cachedPath = currentPath
-	h.cmdCacheMu.Unlock()
-
-	return exists
+	return h.lookupCommandInPath(cmd, currentPath)
 }
 
 // lookupCommandInPath checks if a command exists in the given PATH.
 func (h *Highlighter) lookupCommandInPath(cmd string, pathEnv string) bool {
 	// If the command contains a path separator, check it directly
 	if strings.Contains(cmd, "/") {
-		info, err := os.Stat(cmd)
+		pathToCheck := cmd
+		if !filepath.IsAbs(cmd) {
+			if wd := h.workingDir(); wd != "" {
+				pathToCheck = filepath.Join(wd, cmd)
+			}
+		}
+
+		info, err := os.Stat(pathToCheck)
 		if err != nil {
 			return false
 		}
@@ -162,10 +147,12 @@ func (h *Highlighter) lookupCommandInPath(cmd string, pathEnv string) bool {
 
 	// Search each directory in PATH
 	for _, dir := range strings.Split(pathEnv, string(os.PathListSeparator)) {
-		if dir == "" {
-			dir = "."
+		baseDir := dir
+		if baseDir == "" {
+			baseDir = "."
 		}
-		path := dir + string(os.PathSeparator) + cmd
+
+		path := filepath.Join(baseDir, cmd)
 		info, err := os.Stat(path)
 		if err != nil {
 			continue
@@ -178,6 +165,20 @@ func (h *Highlighter) lookupCommandInPath(cmd string, pathEnv string) bool {
 	return false
 }
 
+func (h *Highlighter) workingDir() string {
+	if h.getWorkingDir != nil {
+		if dir := h.getWorkingDir(); dir != "" {
+			return dir
+		}
+	}
+
+	if dir, err := os.Getwd(); err == nil {
+		return dir
+	}
+
+	return ""
+}
+
 // variableHasValue checks if an environment variable has a non-empty value.
 // It uses the shell's environment if available, falling back to the OS environment.
 func (h *Highlighter) variableHasValue(name string) bool {
@@ -239,13 +240,6 @@ func (h *Highlighter) extractVariableName(varText string) string {
 	return name.String()
 }
 
-// ClearCache clears the command existence cache.
-func (h *Highlighter) ClearCache() {
-	h.cmdCacheMu.Lock()
-	h.cmdCache = make(map[string]bool)
-	h.cmdCacheMu.Unlock()
-}
-
 // Highlight returns the input text with syntax highlighting applied.
 func (h *Highlighter) Highlight(input string) string {
 	if input == "" {
