@@ -7,105 +7,56 @@ import (
 	"testing"
 	"time"
 
-	"github.com/atinylittleshell/gsh/internal/history"
 	"github.com/atinylittleshell/gsh/internal/script/interpreter"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 )
 
-// mockHistoryProvider implements HistoryProvider for testing.
-type mockHistoryProvider struct {
-	entries map[string][]history.HistoryEntry
-	err     error
+// mockProvider implements PredictionProvider for testing.
+type mockProvider struct {
+	instantPredictions   map[string]string
+	debouncedPredictions map[string]string
+	err                  error
 }
 
-func (m *mockHistoryProvider) GetRecentEntriesByPrefix(prefix string, limit int) ([]history.HistoryEntry, error) {
-	if m.err != nil {
-		return nil, m.err
-	}
-	entries, ok := m.entries[prefix]
-	if !ok {
-		return nil, nil
-	}
-	if len(entries) > limit {
-		return entries[:limit], nil
-	}
-	return entries, nil
-}
-
-// mockPredictionProvider implements PredictionProvider for testing.
-type mockPredictionProvider struct {
-	predictions map[string]string
-	err         error
-	callCount   int
-	mu          sync.Mutex
-}
-
-func (m *mockPredictionProvider) Predict(ctx context.Context, input string) (string, error) {
-	m.mu.Lock()
-	m.callCount++
-	m.mu.Unlock()
-
+func (m *mockProvider) Predict(ctx context.Context, input string, trigger interpreter.PredictTrigger, existingPrediction string) (string, error) {
 	if m.err != nil {
 		return "", m.err
 	}
-	prediction, ok := m.predictions[input]
+	if trigger == interpreter.PredictTriggerInstant {
+		prediction, ok := m.instantPredictions[input]
+		if !ok {
+			return "", nil
+		}
+		return prediction, nil
+	}
+	// Debounced trigger
+	prediction, ok := m.debouncedPredictions[input]
 	if !ok {
 		return "", nil
 	}
 	return prediction, nil
 }
 
-func (m *mockPredictionProvider) getCallCount() int {
-	m.mu.Lock()
-	defer m.mu.Unlock()
-	return m.callCount
-}
-
-func TestPredictionSource_String(t *testing.T) {
-	tests := []struct {
-		source   PredictionSource
-		expected string
-	}{
-		{PredictionSourceNone, "none"},
-		{PredictionSourceHistory, "history"},
-		{PredictionSourceLLM, "llm"},
-		{PredictionSource(99), "unknown"},
-	}
-
-	for _, tt := range tests {
-		t.Run(tt.expected, func(t *testing.T) {
-			assert.Equal(t, tt.expected, tt.source.String())
-		})
-	}
-}
-
 func TestNewPredictionState(t *testing.T) {
 	t.Run("default values", func(t *testing.T) {
 		ps := NewPredictionState(PredictionStateConfig{})
 
 		assert.Equal(t, 200*time.Millisecond, ps.debounceDelay)
-		assert.Equal(t, 10, ps.historyPrefixLimit)
 		assert.NotNil(t, ps.logger)
-		assert.Nil(t, ps.historyProvider)
-		assert.Nil(t, ps.llmProvider)
+		assert.Nil(t, ps.provider)
 	})
 
 	t.Run("custom values", func(t *testing.T) {
-		historyProvider := &mockHistoryProvider{}
-		llmProvider := &mockPredictionProvider{}
+		provider := &mockProvider{}
 
 		ps := NewPredictionState(PredictionStateConfig{
-			DebounceDelay:      100 * time.Millisecond,
-			HistoryPrefixLimit: 5,
-			HistoryProvider:    historyProvider,
-			LLMProvider:        llmProvider,
+			DebounceDelay: 100 * time.Millisecond,
+			Provider:      provider,
 		})
 
 		assert.Equal(t, 100*time.Millisecond, ps.debounceDelay)
-		assert.Equal(t, 5, ps.historyPrefixLimit)
-		assert.Equal(t, historyProvider, ps.historyProvider)
-		assert.Equal(t, llmProvider, ps.llmProvider)
+		assert.Equal(t, provider, ps.provider)
 	})
 }
 
@@ -195,24 +146,47 @@ func TestPredictionState_OnInputChanged(t *testing.T) {
 		assert.Equal(t, initialID+1, ps.StateID())
 	})
 
-	t.Run("keeps prediction when input matches prefix", func(t *testing.T) {
+	t.Run("passes existing prediction to provider", func(t *testing.T) {
+		// Now the provider (middleware) decides whether to keep the prediction
+		// The Go code always calls the provider with the existing prediction
+		provider := &mockProvider{
+			instantPredictions: map[string]string{
+				"git": "git status", // Provider returns same prediction
+			},
+		}
+
 		ps := NewPredictionState(PredictionStateConfig{
 			DebounceDelay: 10 * time.Millisecond,
+			Provider:      provider,
 		})
 
 		ps.SetPrediction(ps.StateID(), "git status")
 		ch := ps.OnInputChanged("git")
-		assert.Nil(t, ch) // no new prediction needed
-		assert.Equal(t, "git status", ps.Prediction())
+		require.NotNil(t, ch) // Provider is always called now
+		result := <-ch
+		assert.Equal(t, "git status", result.Prediction)
 	})
 
-	t.Run("clears prediction when input does not match", func(t *testing.T) {
+	t.Run("clears prediction when provider returns empty", func(t *testing.T) {
+		// Provider decides whether to keep or clear prediction
+		// If provider returns empty for a mismatched input, prediction is cleared
+		provider := &mockProvider{
+			instantPredictions:   map[string]string{},
+			debouncedPredictions: map[string]string{},
+		}
+
 		ps := NewPredictionState(PredictionStateConfig{
 			DebounceDelay: 10 * time.Millisecond,
+			Provider:      provider,
 		})
 
 		ps.SetPrediction(ps.StateID(), "git status")
-		ps.OnInputChanged("ls")
+		ch := ps.OnInputChanged("ls")
+		// Wait for debounced result (instant returns empty, so falls through to debounced)
+		if ch != nil {
+			<-ch
+		}
+		// No prediction returned from provider, so it stays empty
 		assert.Equal(t, "", ps.Prediction())
 	})
 
@@ -229,19 +203,16 @@ func TestPredictionState_OnInputChanged(t *testing.T) {
 	})
 }
 
-func TestPredictionState_HistoryPrediction(t *testing.T) {
-	historyProvider := &mockHistoryProvider{
-		entries: map[string][]history.HistoryEntry{
-			"git": {
-				{Command: "git status"},
-				{Command: "git log"},
-			},
+func TestPredictionState_InstantPrediction(t *testing.T) {
+	provider := &mockProvider{
+		instantPredictions: map[string]string{
+			"git": "git status",
 		},
 	}
 
 	ps := NewPredictionState(PredictionStateConfig{
-		DebounceDelay:   10 * time.Millisecond,
-		HistoryProvider: historyProvider,
+		DebounceDelay: 10 * time.Millisecond,
+		Provider:      provider,
 	})
 
 	ch := ps.OnInputChanged("git")
@@ -251,30 +222,27 @@ func TestPredictionState_HistoryPrediction(t *testing.T) {
 	select {
 	case result := <-ch:
 		assert.Equal(t, "git status", result.Prediction)
-		assert.Equal(t, PredictionSourceHistory, result.Source)
 		assert.NoError(t, result.Error)
 	case <-time.After(500 * time.Millisecond):
 		t.Fatal("timeout waiting for prediction")
 	}
 }
 
-func TestPredictionState_HistoryPredictionIsInstant(t *testing.T) {
-	// This test verifies that history-based predictions bypass the debounce delay
+func TestPredictionState_InstantPredictionIsInstant(t *testing.T) {
+	// This test verifies that instant predictions bypass the debounce delay
 	// and return immediately (synchronously).
 
-	historyProvider := &mockHistoryProvider{
-		entries: map[string][]history.HistoryEntry{
-			"git": {
-				{Command: "git status"},
-			},
+	provider := &mockProvider{
+		instantPredictions: map[string]string{
+			"git": "git status",
 		},
 	}
 
 	// Use a very long debounce delay to make the test obvious
-	// If history predictions were debounced, this test would timeout
+	// If instant predictions were debounced, this test would timeout
 	ps := NewPredictionState(PredictionStateConfig{
-		DebounceDelay:   10 * time.Second, // Very long debounce
-		HistoryProvider: historyProvider,
+		DebounceDelay: 10 * time.Second, // Very long debounce
+		Provider:      provider,
 	})
 
 	start := time.Now()
@@ -288,37 +256,32 @@ func TestPredictionState_HistoryPredictionIsInstant(t *testing.T) {
 
 		// Should complete in under 10ms (well under the 10s debounce)
 		assert.Less(t, elapsed, 10*time.Millisecond,
-			"history prediction should be instant, not debounced")
+			"instant prediction should be instant, not debounced")
 
 		assert.Equal(t, "git status", result.Prediction)
-		assert.Equal(t, PredictionSourceHistory, result.Source)
 
 		// Verify prediction was also set synchronously on the state
 		assert.Equal(t, "git status", ps.Prediction())
 	case <-time.After(100 * time.Millisecond):
-		t.Fatal("history prediction should be instant, but timed out")
+		t.Fatal("instant prediction should be instant, but timed out")
 	}
 }
 
-func TestPredictionState_LLMPredictionIsDebounced(t *testing.T) {
-	// This test verifies that LLM predictions (when no history match) ARE debounced
+func TestPredictionState_DebouncedPrediction(t *testing.T) {
+	// This test verifies that debounced predictions (when no instant match) ARE debounced
 
-	// Empty history - no matches
-	historyProvider := &mockHistoryProvider{
-		entries: map[string][]history.HistoryEntry{},
-	}
-
-	llmProvider := &mockPredictionProvider{
-		predictions: map[string]string{
+	// Provider with no instant matches but debounced match
+	provider := &mockProvider{
+		instantPredictions: map[string]string{},
+		debouncedPredictions: map[string]string{
 			"docker": "docker ps",
 		},
 	}
 
 	debounceDelay := 50 * time.Millisecond
 	ps := NewPredictionState(PredictionStateConfig{
-		DebounceDelay:   debounceDelay,
-		HistoryProvider: historyProvider,
-		LLMProvider:     llmProvider,
+		DebounceDelay: debounceDelay,
+		Provider:      provider,
 	})
 
 	start := time.Now()
@@ -335,28 +298,23 @@ func TestPredictionState_LLMPredictionIsDebounced(t *testing.T) {
 			"LLM prediction should be debounced")
 
 		assert.Equal(t, "docker ps", result.Prediction)
-		assert.Equal(t, PredictionSourceLLM, result.Source)
 	case <-time.After(500 * time.Millisecond):
 		t.Fatal("timeout waiting for LLM prediction")
 	}
 }
 
-func TestPredictionState_LLMFallback(t *testing.T) {
-	// Empty history provider
-	historyProvider := &mockHistoryProvider{
-		entries: map[string][]history.HistoryEntry{},
-	}
-
-	llmProvider := &mockPredictionProvider{
-		predictions: map[string]string{
+func TestPredictionState_DebouncedFallback(t *testing.T) {
+	// No instant matches, should fall back to debounced prediction
+	provider := &mockProvider{
+		instantPredictions: map[string]string{},
+		debouncedPredictions: map[string]string{
 			"docker": "docker ps",
 		},
 	}
 
 	ps := NewPredictionState(PredictionStateConfig{
-		DebounceDelay:   10 * time.Millisecond,
-		HistoryProvider: historyProvider,
-		LLMProvider:     llmProvider,
+		DebounceDelay: 10 * time.Millisecond,
+		Provider:      provider,
 	})
 
 	ch := ps.OnInputChanged("docker")
@@ -366,7 +324,6 @@ func TestPredictionState_LLMFallback(t *testing.T) {
 	select {
 	case result := <-ch:
 		assert.Equal(t, "docker ps", result.Prediction)
-		assert.Equal(t, PredictionSourceLLM, result.Source)
 		assert.NoError(t, result.Error)
 	case <-time.After(500 * time.Millisecond):
 		t.Fatal("timeout waiting for prediction")
@@ -374,15 +331,15 @@ func TestPredictionState_LLMFallback(t *testing.T) {
 }
 
 func TestPredictionState_NullStatePrediction(t *testing.T) {
-	llmProvider := &mockPredictionProvider{
-		predictions: map[string]string{
+	provider := &mockProvider{
+		debouncedPredictions: map[string]string{
 			"": "ls -la",
 		},
 	}
 
 	ps := NewPredictionState(PredictionStateConfig{
 		DebounceDelay: 10 * time.Millisecond,
-		LLMProvider:   llmProvider,
+		Provider:      provider,
 	})
 
 	// Mark dirty first, then clear
@@ -394,33 +351,31 @@ func TestPredictionState_NullStatePrediction(t *testing.T) {
 	select {
 	case result := <-ch:
 		assert.Equal(t, "ls -la", result.Prediction)
-		assert.Equal(t, PredictionSourceLLM, result.Source)
 	case <-time.After(500 * time.Millisecond):
 		t.Fatal("timeout waiting for prediction")
 	}
 }
 
 func TestPredictionState_AgentChatSkipped(t *testing.T) {
-	t.Run("LLM prediction skipped for agent commands", func(t *testing.T) {
-		llmProvider := &mockPredictionProvider{
-			predictions: map[string]string{
+	t.Run("debounced prediction skipped for agent commands", func(t *testing.T) {
+		provider := &mockProvider{
+			debouncedPredictions: map[string]string{
 				"#hello": "should not appear",
 			},
 		}
 
 		ps := NewPredictionState(PredictionStateConfig{
 			DebounceDelay: 10 * time.Millisecond,
-			LLMProvider:   llmProvider,
+			Provider:      provider,
 		})
 
 		ch := ps.OnInputChanged("#hello")
-		// LLM prediction is started but will return empty for agent chat messages
+		// Debounced prediction is started but will return empty for agent chat messages
 		if ch != nil {
 			select {
 			case result := <-ch:
-				// LLM should return empty prediction for agent commands
+				// Should return empty prediction for agent commands
 				assert.Equal(t, "", result.Prediction)
-				assert.Equal(t, PredictionSourceNone, result.Source)
 			case <-time.After(500 * time.Millisecond):
 				t.Fatal("timeout waiting for prediction result")
 			}
@@ -433,29 +388,25 @@ func TestPredictionState_AgentChatSkipped(t *testing.T) {
 		// Note: LLM is called but skips prediction internally for # commands
 	})
 
-	t.Run("history prediction works for agent commands", func(t *testing.T) {
-		historyProvider := &mockHistoryProvider{
-			entries: map[string][]history.HistoryEntry{
-				"#": {
-					{Command: "#explain this code"},
-					{Command: "#help"},
-				},
+	t.Run("instant prediction works for agent commands", func(t *testing.T) {
+		provider := &mockProvider{
+			instantPredictions: map[string]string{
+				"#": "#explain this code",
 			},
 		}
 
 		ps := NewPredictionState(PredictionStateConfig{
-			DebounceDelay:   10 * time.Millisecond,
-			HistoryProvider: historyProvider,
+			DebounceDelay: 10 * time.Millisecond,
+			Provider:      provider,
 		})
 
 		ch := ps.OnInputChanged("#")
-		require.NotNil(t, ch, "history prediction should return a channel for agent commands")
+		require.NotNil(t, ch, "instant prediction should return a channel for agent commands")
 
 		select {
 		case result := <-ch:
-			// History prediction should work for agent commands
+			// Instant prediction should work for agent commands
 			assert.Equal(t, "#explain this code", result.Prediction)
-			assert.Equal(t, PredictionSourceHistory, result.Source)
 		case <-time.After(100 * time.Millisecond):
 			t.Fatal("timeout waiting for history prediction")
 		}
@@ -466,15 +417,15 @@ func TestPredictionState_AgentChatSkipped(t *testing.T) {
 }
 
 func TestPredictionState_Debouncing(t *testing.T) {
-	llmProvider := &mockPredictionProvider{
-		predictions: map[string]string{
+	provider := &mockProvider{
+		debouncedPredictions: map[string]string{
 			"final": "final command",
 		},
 	}
 
 	ps := NewPredictionState(PredictionStateConfig{
 		DebounceDelay: 50 * time.Millisecond,
-		LLMProvider:   llmProvider,
+		Provider:      provider,
 	})
 
 	// Rapid input changes
@@ -492,14 +443,11 @@ func TestPredictionState_Debouncing(t *testing.T) {
 	case <-time.After(500 * time.Millisecond):
 		t.Fatal("timeout waiting for prediction")
 	}
-
-	// Only one actual LLM call should have been made
-	assert.Equal(t, 1, llmProvider.getCallCount())
 }
 
 func TestPredictionState_CancellationOnNewInput(t *testing.T) {
-	slowProvider := &mockPredictionProvider{
-		predictions: map[string]string{
+	provider := &mockProvider{
+		debouncedPredictions: map[string]string{
 			"slow": "slow result",
 			"fast": "fast result",
 		},
@@ -507,7 +455,7 @@ func TestPredictionState_CancellationOnNewInput(t *testing.T) {
 
 	ps := NewPredictionState(PredictionStateConfig{
 		DebounceDelay: 10 * time.Millisecond,
-		LLMProvider:   slowProvider,
+		Provider:      provider,
 	})
 
 	// Start first prediction
@@ -527,14 +475,14 @@ func TestPredictionState_CancellationOnNewInput(t *testing.T) {
 	}
 }
 
-func TestPredictionState_LLMError(t *testing.T) {
-	llmProvider := &mockPredictionProvider{
-		err: errors.New("LLM error"),
+func TestPredictionState_ProviderError(t *testing.T) {
+	provider := &mockProvider{
+		err: errors.New("provider error"),
 	}
 
 	ps := NewPredictionState(PredictionStateConfig{
 		DebounceDelay: 10 * time.Millisecond,
-		LLMProvider:   llmProvider,
+		Provider:      provider,
 	})
 
 	ch := ps.OnInputChanged("test")
@@ -552,7 +500,7 @@ func TestPredictionState_LLMError(t *testing.T) {
 func TestLLMPredictionProvider(t *testing.T) {
 	t.Run("nil model returns empty", func(t *testing.T) {
 		provider := NewLLMPredictionProvider(nil, nil, nil)
-		prediction, err := provider.Predict(context.Background(), "test")
+		prediction, err := provider.Predict(context.Background(), "test", interpreter.PredictTriggerDebounced, "")
 		assert.NoError(t, err)
 		assert.Equal(t, "", prediction)
 	})
@@ -567,15 +515,6 @@ func TestLLMPredictionProvider(t *testing.T) {
 	})
 }
 
-func TestHistoryPredictionAdapter(t *testing.T) {
-	t.Run("nil manager returns nil", func(t *testing.T) {
-		adapter := NewHistoryPredictionAdapter(nil)
-		entries, err := adapter.GetRecentEntriesByPrefix("git", 10)
-		assert.NoError(t, err)
-		assert.Nil(t, entries)
-	})
-}
-
 // mockModelProvider implements interpreter.ModelProvider for testing.
 type mockModelProvider struct {
 	response *interpreter.ChatResponse
@@ -586,15 +525,15 @@ func (m *mockModelProvider) Name() string {
 	return "mock"
 }
 
-func (m *mockModelProvider) ChatCompletion(request interpreter.ChatRequest) (*interpreter.ChatResponse, error) {
+func (m *mockModelProvider) ChatCompletion(ctx context.Context, request interpreter.ChatRequest) (*interpreter.ChatResponse, error) {
 	if m.err != nil {
 		return nil, m.err
 	}
 	return m.response, nil
 }
 
-func (m *mockModelProvider) StreamingChatCompletion(request interpreter.ChatRequest, callbacks *interpreter.StreamCallbacks) (*interpreter.ChatResponse, error) {
-	response, err := m.ChatCompletion(request)
+func (m *mockModelProvider) StreamingChatCompletion(ctx context.Context, request interpreter.ChatRequest, callbacks *interpreter.StreamCallbacks) (*interpreter.ChatResponse, error) {
+	response, err := m.ChatCompletion(ctx, request)
 	if err != nil {
 		return nil, err
 	}
@@ -618,7 +557,7 @@ func TestLLMPredictionProvider_WithMockProvider(t *testing.T) {
 		}
 
 		provider := NewLLMPredictionProvider(model, mockProvider, nil)
-		prediction, err := provider.Predict(context.Background(), "git")
+		prediction, err := provider.Predict(context.Background(), "git", interpreter.PredictTriggerDebounced, "")
 
 		assert.NoError(t, err)
 		assert.Equal(t, "git status", prediction)
@@ -632,7 +571,7 @@ func TestLLMPredictionProvider_WithMockProvider(t *testing.T) {
 		}
 
 		provider := NewLLMPredictionProvider(model, mockProvider, nil)
-		prediction, err := provider.Predict(context.Background(), "")
+		prediction, err := provider.Predict(context.Background(), "", interpreter.PredictTriggerDebounced, "")
 
 		assert.NoError(t, err)
 		assert.Equal(t, "ls -la", prediction)
@@ -644,7 +583,7 @@ func TestLLMPredictionProvider_WithMockProvider(t *testing.T) {
 		}
 
 		provider := NewLLMPredictionProvider(model, mockProvider, nil)
-		_, err := provider.Predict(context.Background(), "test")
+		_, err := provider.Predict(context.Background(), "test", interpreter.PredictTriggerDebounced, "")
 
 		assert.Error(t, err)
 	})
@@ -657,7 +596,7 @@ func TestLLMPredictionProvider_WithMockProvider(t *testing.T) {
 		}
 
 		provider := NewLLMPredictionProvider(model, mockProvider, nil)
-		prediction, err := provider.Predict(context.Background(), "test")
+		prediction, err := provider.Predict(context.Background(), "test", interpreter.PredictTriggerDebounced, "")
 
 		assert.NoError(t, err)
 		assert.Equal(t, "", prediction)
