@@ -8,8 +8,11 @@ import (
 	"fmt"
 	"io/fs"
 	"os"
+	"os/signal"
 	"path/filepath"
 	"strings"
+	"sync/atomic"
+	"syscall"
 	"time"
 
 	tea "github.com/charmbracelet/bubbletea"
@@ -21,7 +24,6 @@ import (
 	"github.com/atinylittleshell/gsh/internal/history"
 	"github.com/atinylittleshell/gsh/internal/repl/completion"
 	"github.com/atinylittleshell/gsh/internal/repl/config"
-	replcontext "github.com/atinylittleshell/gsh/internal/repl/context"
 	"github.com/atinylittleshell/gsh/internal/repl/executor"
 	"github.com/atinylittleshell/gsh/internal/repl/input"
 	"github.com/atinylittleshell/gsh/internal/repl/predict"
@@ -39,8 +41,7 @@ type REPL struct {
 	config             *config.Config
 	executor           *executor.REPLExecutor
 	history            *history.HistoryManager
-	predictor          *predict.Router
-	contextProvider    *replcontext.Provider
+	predictor          input.PredictionProvider
 	completionProvider *completion.Provider
 	logger             *zap.Logger
 
@@ -51,6 +52,8 @@ type REPL struct {
 	// Startup tracking
 	startTime      time.Time
 	startupTracker StartupTimeTracker
+
+	sigintChannelFactory func() (chan os.Signal, func())
 }
 
 // Options holds configuration options for creating a new REPL.
@@ -211,26 +214,15 @@ func NewREPL(opts Options) (*REPL, error) {
 		// Continue without history - not fatal
 	}
 
-	// Initialize prediction router using lazy model resolution.
-	// Use SDKModelRef so that predictions always use the current gsh.models.lite value,
-	// even if the user changes it after startup (e.g., in an event handler).
-	liteModelRef := &interpreter.SDKModelRef{Tier: "lite", Models: interp.SDKConfig().GetModels()}
-
-	// Initialize prediction router using the lite model tier with lazy resolution
-	predictor := predict.NewRouterFromConfig(liteModelRef, logger)
-
-	// Initialize context provider with retrievers for predictions
-	contextProvider := replcontext.NewProvider(logger,
-		replcontext.NewWorkingDirectoryRetriever(exec),
-		replcontext.NewGitStatusRetriever(exec, logger),
-		replcontext.NewSystemInfoRetriever(),
-	)
-
-	// Add history retriever if history is available
+	// Wire up history provider for gsh.history SDK access
 	if historyMgr != nil {
-		contextProvider.AddRetriever(replcontext.NewConciseHistoryRetriever(historyMgr, 0))
+		interp.SDKConfig().SetHistoryProvider(&historyProviderAdapter{manager: historyMgr})
 	}
 
+	// Event-driven prediction provider that delegates to gsh middleware (repl.predict event).
+	// The middleware in cmd/gsh/defaults/middleware/prediction.gsh handles context gathering internally.
+	eventPredictor := predict.NewEventPredictionProvider(interp, logger)
+
 	// Initialize completion provider
 	completionProvider := completion.NewProvider(exec)
 
@@ -238,23 +230,41 @@ func NewREPL(opts Options) (*REPL, error) {
 		config:             loadResult.Config,
 		executor:           exec,
 		history:            historyMgr,
-		predictor:          predictor,
-		contextProvider:    contextProvider,
+		predictor:          eventPredictor,
 		completionProvider: completionProvider,
 		logger:             logger,
 		startTime:          opts.StartTime,
 		startupTracker:     opts.StartupTracker,
+		sigintChannelFactory: func() (chan os.Signal, func()) {
+			ch := make(chan os.Signal, 1)
+			signal.Notify(ch, syscall.SIGINT)
+			return ch, func() { signal.Stop(ch) }
+		},
 	}
 
 	return repl, nil
 }
 
+func (r *REPL) newSigintChannel() (chan os.Signal, func()) {
+	if r.sigintChannelFactory != nil {
+		return r.sigintChannelFactory()
+	}
+
+	ch := make(chan os.Signal, 1)
+	signal.Notify(ch, syscall.SIGINT)
+	return ch, func() { signal.Stop(ch) }
+}
+
+func (r *REPL) setSigintChannelFactory(factory func() (chan os.Signal, func())) {
+	r.sigintChannelFactory = factory
+}
+
 // Run starts the interactive REPL loop.
 func (r *REPL) Run(ctx context.Context) error {
 	r.logger.Info("starting REPL")
 
 	// Emit repl.ready event (welcome screen is handled by event handler in defaults/events/repl.gsh)
-	r.emitREPLEvent("repl.ready")
+	r.executor.Interpreter().EmitEvent(interpreter.EventReplReady, interpreter.CreateReplReadyContext())
 
 	// Track startup time - this is when the user actually sees the welcome screen
 	if r.startupTracker != nil && !r.startTime.IsZero() {
@@ -262,26 +272,14 @@ func (r *REPL) Run(ctx context.Context) error {
 		r.startupTracker.TrackStartupTime(startupMs)
 	}
 
-	// Create prediction state if history or LLM predictor is available
-	// History-based prediction doesn't require an LLM model
+	// Create prediction state if predictor is available
+	// The predictor handles both instant (history) and debounced (LLM) predictions
+	// via the repl.predict event with different trigger types.
 	var predictionState *input.PredictionState
-	var historyProvider input.HistoryProvider
-	if r.history != nil {
-		historyProvider = input.NewHistoryPredictionAdapter(r.history)
-	}
-
-	// Create prediction state if we have history or LLM predictor
-	if historyProvider != nil || r.predictor != nil {
-		// Only set LLMProvider if predictor is not nil to avoid nil interface issues
-		var llmProvider input.PredictionProvider
-		if r.predictor != nil {
-			llmProvider = r.predictor
-		}
-
+	if r.predictor != nil {
 		predictionState = input.NewPredictionState(input.PredictionStateConfig{
-			HistoryProvider: historyProvider,
-			LLMProvider:     llmProvider,
-			Logger:          r.logger,
+			Provider: r.predictor,
+			Logger:   r.logger,
 		})
 	}
 
@@ -289,7 +287,7 @@ func (r *REPL) Run(ctx context.Context) error {
 		// Check context cancellation
 		select {
 		case <-ctx.Done():
-			r.emitREPLEvent("repl.exit")
+			r.executor.Interpreter().EmitEvent(interpreter.EventReplExit, interpreter.CreateReplExitContext())
 			return ctx.Err()
 		default:
 		}
@@ -317,14 +315,12 @@ func (r *REPL) Run(ctx context.Context) error {
 			CompletionProvider: r.completionProvider,
 			AliasExistsFunc:    r.executor.AliasOrFunctionExists,
 			GetEnvFunc:         r.executor.GetEnv,
+			GetWorkingDirFunc:  r.executor.GetPwd,
 			PredictionState:    predictionState,
 			Width:              termWidth,
 			Logger:             r.logger,
 		})
 
-		// Update predictor context asynchronously (don't block prompt display)
-		go r.updatePredictorContext()
-
 		// Run the input loop
 		p := tea.NewProgram(inputModel,
 			tea.WithContext(ctx),
@@ -368,7 +364,7 @@ func (r *REPL) Run(ctx context.Context) error {
 			if err := r.processCommand(ctx, result.Value); err != nil {
 				// Check if user requested exit
 				if err == ErrExit {
-					r.emitREPLEvent("repl.exit")
+					r.executor.Interpreter().EmitEvent(interpreter.EventReplExit, interpreter.CreateReplExitContext())
 					return nil
 				}
 				// Log other errors but continue
@@ -388,6 +384,45 @@ func (r *REPL) processCommand(ctx context.Context, command string) error {
 		return nil
 	}
 
+	// Set up SIGINT handling for command execution
+	// Create a cancellable context that will be cancelled on Ctrl+C
+	cmdCtx, cancel := context.WithCancel(ctx)
+	defer cancel()
+
+	// Set up signal handling for SIGINT (Ctrl+C)
+	sigChan, stopSigint := r.newSigintChannel()
+	defer stopSigint()
+
+	// Track if we were interrupted
+	interrupted := false
+	var ignoreSigint atomic.Bool
+
+	// Start a goroutine to handle SIGINT
+	go func() {
+		for {
+			select {
+			case <-sigChan:
+				// When a foreground child owns the terminal, let it handle SIGINT.
+				if ignoreSigint.Load() {
+					continue
+				}
+				// Ctrl+C received - cancel the context
+				// Note: The terminal echoes ^C automatically, so we don't print it here
+				interrupted = true
+				cancel()
+				return
+			case <-cmdCtx.Done():
+				// Context was cancelled for another reason, exit goroutine
+				return
+			}
+		}
+	}()
+
+	// Set the cancellable context on the interpreter so agent execution can use it
+	interp := r.executor.Interpreter()
+	interp.SetContext(cmdCtx)
+	defer interp.SetContext(context.Background()) // Clear context after command completes
+
 	// Record ALL user input in history (including agent commands like "#...")
 	// This is done before middleware so all user input is captured
 	var historyEntry *history.HistoryEntry
@@ -400,34 +435,54 @@ func (r *REPL) processCommand(ctx context.Context, command string) error {
 		}
 	}
 
-	// Try middleware chain first
-	interp := r.executor.Interpreter()
-	replCtx := interp.SDKConfig().GetREPLContext()
-	if replCtx != nil && replCtx.MiddlewareManager != nil && replCtx.MiddlewareManager.Len() > 0 {
-		result, err := replCtx.MiddlewareManager.ExecuteChain(command, interp)
-		if err != nil {
-			r.logger.Debug("middleware error", zap.Error(err))
-			fmt.Fprintf(os.Stderr, "gsh: middleware error: %v\n", err)
-			// Finish history with error exit code
-			if historyEntry != nil {
-				if _, finishErr := r.history.FinishCommand(historyEntry, 1); finishErr != nil {
-					r.logger.Debug("failed to finish history entry", zap.Error(finishErr))
+	// Emit command.input event to middleware chain
+	inputCtx := &interpreter.ObjectValue{
+		Properties: map[string]*interpreter.PropertyDescriptor{
+			"input": {Value: &interpreter.StringValue{Value: command}},
+		},
+	}
+	result := interp.EmitEvent("command.input", inputCtx)
+
+	// Check if we were interrupted during middleware execution
+	if interrupted {
+		if historyEntry != nil {
+			// Record as interrupted (exit code 130 is standard for SIGINT)
+			if _, finishErr := r.history.FinishCommand(historyEntry, 130); finishErr != nil {
+				r.logger.Debug("failed to finish history entry", zap.Error(finishErr))
+			}
+		}
+		return nil
+	}
+
+	// Check if middleware handled the command
+	if result != nil {
+		if obj, ok := result.(*interpreter.ObjectValue); ok {
+			// Check for { handled: true }
+			if handledVal := obj.GetPropertyValue("handled"); handledVal != nil {
+				if bv, ok := handledVal.(*interpreter.BoolValue); ok && bv.Value {
+					// Middleware handled the input, don't execute as shell command
+					if historyEntry != nil {
+						if _, finishErr := r.history.FinishCommand(historyEntry, 0); finishErr != nil {
+							r.logger.Debug("failed to finish history entry", zap.Error(finishErr))
+						}
+					}
+					return nil
 				}
 			}
-			return nil
-		}
-		if result.Handled {
-			// Middleware handled the input, don't execute as shell command
-			// Finish history with success exit code
-			if historyEntry != nil {
-				if _, finishErr := r.history.FinishCommand(historyEntry, 0); finishErr != nil {
-					r.logger.Debug("failed to finish history entry", zap.Error(finishErr))
+			// Check for modified input
+			if inputVal := obj.GetPropertyValue("input"); inputVal != nil {
+				if sv, ok := inputVal.(*interpreter.StringValue); ok {
+					command = sv.Value
 				}
 			}
-			return nil
 		}
-		// Use potentially modified input from middleware
-		command = result.Input
+	}
+
+	// Get potentially modified input from context (middleware may have modified it)
+	if inputVal := inputCtx.GetPropertyValue("input"); inputVal != nil {
+		if sv, ok := inputVal.(*interpreter.StringValue); ok {
+			command = sv.Value
+		}
 	}
 
 	// Handle built-in commands (like "exit")
@@ -441,7 +496,9 @@ func (r *REPL) processCommand(ctx context.Context, command string) error {
 	}
 
 	// Fall through: execute as shell command
-	exitCode := r.executeShellCommand(ctx, command)
+	ignoreSigint.Store(true)
+	exitCode := r.executeShellCommand(cmdCtx, command)
+	ignoreSigint.Store(false)
 
 	// Finish history entry with actual exit code
 	if historyEntry != nil {
@@ -460,7 +517,7 @@ func (r *REPL) processCommand(ctx context.Context, command string) error {
 // Returns the exit code of the command.
 func (r *REPL) executeShellCommand(ctx context.Context, command string) int {
 	// Emit repl.command.before event with the command text
-	r.emitREPLEvent("repl.command.before", &interpreter.StringValue{Value: command})
+	r.executor.Interpreter().EmitEvent(interpreter.EventReplCommandBefore, interpreter.CreateReplCommandBeforeContext(command))
 
 	// Execute the command
 	startTime := timeNow()
@@ -472,13 +529,10 @@ func (r *REPL) executeShellCommand(ctx context.Context, command string) int {
 	r.lastDurationMs = duration.Milliseconds()
 
 	// Update the interpreter's REPL context with the last command info
-	r.executor.Interpreter().SDKConfig().UpdateLastCommand(exitCode, r.lastDurationMs)
+	r.executor.Interpreter().SDKConfig().UpdateLastCommand(command, exitCode, r.lastDurationMs)
 
 	// Emit repl.command.after event with command, exit code, and duration
-	r.emitREPLEvent("repl.command.after",
-		&interpreter.StringValue{Value: command},
-		&interpreter.NumberValue{Value: float64(exitCode)},
-		&interpreter.NumberValue{Value: float64(r.lastDurationMs)})
+	r.executor.Interpreter().EmitEvent(interpreter.EventReplCommandAfter, interpreter.CreateReplCommandAfterContext(command, exitCode, r.lastDurationMs))
 
 	// Display error if execution failed (not just non-zero exit)
 	if err != nil {
@@ -508,7 +562,7 @@ func (r *REPL) getPrompt() string {
 	interp := r.executor.Interpreter()
 
 	// Emit repl.prompt event to let handlers update the prompt dynamically
-	interp.EmitEvent("repl.prompt", &interpreter.NullValue{})
+	interp.EmitEvent(interpreter.EventReplPrompt, interpreter.CreateReplPromptContext())
 
 	// Read gsh.prompt property (may have been updated by event handler)
 	replCtx := interp.SDKConfig().GetREPLContext()
@@ -522,16 +576,6 @@ func (r *REPL) getPrompt() string {
 	return "gsh> "
 }
 
-// updatePredictorContext updates the predictor with current context information.
-func (r *REPL) updatePredictorContext() {
-	if r.predictor == nil || r.contextProvider == nil {
-		return
-	}
-
-	contextMap := r.contextProvider.GetContext()
-	r.predictor.UpdateContext(contextMap)
-}
-
 // getHistoryValues returns recent history entries for navigation.
 func (r *REPL) getHistoryValues() []string {
 	if r.history == nil {
@@ -599,20 +643,62 @@ func (r *REPL) History() *history.HistoryManager {
 	return r.history
 }
 
-// emitREPLEvent emits a REPL event by calling all registered handlers for that event
-func (r *REPL) emitREPLEvent(eventName string, args ...interpreter.Value) {
-	interp := r.executor.Interpreter()
-	handlers := interp.GetEventHandlers(eventName)
-	for _, handler := range handlers {
-		// Call each handler with the provided arguments
-		// Errors are logged but don't stop other handlers
-		if _, err := interp.CallTool(handler, args); err != nil {
-			r.logger.Debug("error in event handler", zap.String("event", eventName), zap.Error(err))
-			// Print errors to stderr for critical events so users can debug
-			// This helps catch issues in ~/.gsh/repl.gsh event handlers
-			fmt.Fprintf(os.Stderr, "gsh: error in %s handler: %v\n", eventName, err)
+// historyProviderAdapter adapts history.HistoryManager to interpreter.HistoryProvider
+type historyProviderAdapter struct {
+	manager *history.HistoryManager
+}
+
+// FindPrefix implements interpreter.HistoryProvider
+func (a *historyProviderAdapter) FindPrefix(prefix string, limit int) ([]interpreter.HistoryEntry, error) {
+	if a.manager == nil {
+		return nil, nil
+	}
+	entries, err := a.manager.GetRecentEntriesByPrefix(prefix, limit)
+	if err != nil {
+		return nil, err
+	}
+	// Convert history.HistoryEntry to interpreter.HistoryEntry
+	result := make([]interpreter.HistoryEntry, len(entries))
+	for i, e := range entries {
+		exitCode := -1 // Default to -1 if exit code is not recorded
+		if e.ExitCode.Valid {
+			exitCode = int(e.ExitCode.Int32)
+		}
+		result[i] = interpreter.HistoryEntry{
+			Command:   e.Command,
+			Timestamp: e.CreatedAt.Unix(),
+			ExitCode:  exitCode,
 		}
 	}
+	return result, nil
+}
+
+// GetRecent implements interpreter.HistoryProvider
+// Returns entries in chronological order (oldest first, most recent last).
+func (a *historyProviderAdapter) GetRecent(limit int) ([]interpreter.HistoryEntry, error) {
+	if a.manager == nil {
+		return nil, nil
+	}
+	// GetRecentEntries returns entries in chronological order (oldest first)
+	entries, err := a.manager.GetRecentEntries("", limit)
+	if err != nil {
+		return nil, err
+	}
+	// Convert history.HistoryEntry to interpreter.HistoryEntry
+	// Keep chronological order (oldest first, most recent last)
+	result := make([]interpreter.HistoryEntry, len(entries))
+	for i, e := range entries {
+		exitCode := -1 // Default to -1 if exit code is not recorded
+		if e.ExitCode.Valid {
+			exitCode = int(e.ExitCode.Int32)
+		}
+		result[i] = interpreter.HistoryEntry{
+			Command:   e.Command,
+			Timestamp: e.CreatedAt.Unix(),
+			ExitCode:  exitCode,
+		}
+	}
+	return result, nil
 }
 
 // loadBashConfigs loads bash configuration files in the correct order.
