@@ -2,7 +2,9 @@ package repl
 
 import (
 	"context"
+	"os"
 	"path/filepath"
+	"syscall"
 	"testing"
 	"time"
 
@@ -21,6 +23,18 @@ import (
 	"github.com/atinylittleshell/gsh/internal/script/interpreter"
 )
 
+// mockPredictionProvider implements input.PredictionProvider for testing.
+type mockPredictionProvider struct {
+	prediction string
+}
+
+func (m *mockPredictionProvider) Predict(ctx context.Context, inputStr string, trigger interpreter.PredictTrigger, existingPrediction string) (string, error) {
+	if trigger == interpreter.PredictTriggerInstant {
+		return m.prediction, nil
+	}
+	return "", nil
+}
+
 func TestDirectoryStructure(t *testing.T) {
 	// This test verifies that all subpackages in internal/repl/ can be imported.
 	// The imports above will fail at compile time if any package is missing
@@ -265,10 +279,11 @@ func TestREPL_GetPrompt(t *testing.T) {
 
 	// Use DefaultConfigContent to set up SDK config with custom prompt via event handler
 	defaultConfig := `
-tool onPrompt(ctx) {
+tool onPrompt(ctx, next) {
 	gsh.prompt = "custom> "
+	return next(ctx)
 }
-gsh.on("repl.prompt", onPrompt)
+gsh.use("repl.prompt", onPrompt)
 `
 
 	repl, err := NewREPL(Options{
@@ -399,7 +414,7 @@ func TestREPL_HandleBuiltinCommand_UnknownCommand(t *testing.T) {
 	assert.NoError(t, err)
 }
 
-func TestREPL_ContextProviderInitialized(t *testing.T) {
+func TestREPL_PredictorInitialized(t *testing.T) {
 	tmpDir := t.TempDir()
 	historyPath := filepath.Join(tmpDir, "history.db")
 	configPath := filepath.Join(tmpDir, "nonexistent.repl.gsh")
@@ -412,142 +427,8 @@ func TestREPL_ContextProviderInitialized(t *testing.T) {
 	require.NoError(t, err)
 	defer repl.Close()
 
-	// Verify context provider was created
-	assert.NotNil(t, repl.contextProvider)
-
-	// Get context and verify it contains expected keys
-	contextMap := repl.contextProvider.GetContext()
-
-	// Should have working_directory
-	_, hasWorkingDir := contextMap["working_directory"]
-	assert.True(t, hasWorkingDir, "context should include working_directory")
-
-	// Should have system_info
-	_, hasSystemInfo := contextMap["system_info"]
-	assert.True(t, hasSystemInfo, "context should include system_info")
-
-	// Should have git_status (might be "not in a git repository")
-	_, hasGitStatus := contextMap["git_status"]
-	assert.True(t, hasGitStatus, "context should include git_status")
-
-	// Should have history_concise (since history manager was initialized)
-	_, hasHistory := contextMap["history_concise"]
-	assert.True(t, hasHistory, "context should include history_concise")
-}
-
-func TestREPL_UpdatePredictorContext_WithLazyModelResolver(t *testing.T) {
-	tmpDir := t.TempDir()
-	historyPath := filepath.Join(tmpDir, "history.db")
-	configPath := filepath.Join(tmpDir, "nonexistent.repl.gsh")
-
-	repl, err := NewREPL(Options{
-		ConfigPath:  configPath,
-		HistoryPath: historyPath,
-		Logger:      zap.NewNop(),
-	})
-	require.NoError(t, err)
-	defer repl.Close()
-
-	// Predictor is now always created with lazy model resolution (SDKModelRef)
-	// It will return empty predictions if gsh.models.lite is not configured
+	// Verify predictor was created (event-driven prediction provider)
 	assert.NotNil(t, repl.predictor)
-
-	// updatePredictorContext should not panic
-	repl.updatePredictorContext()
-}
-
-func TestREPL_UpdatePredictorContext_NilContextProvider(t *testing.T) {
-	tmpDir := t.TempDir()
-	historyPath := filepath.Join(tmpDir, "history.db")
-	configPath := filepath.Join(tmpDir, "nonexistent.repl.gsh")
-
-	repl, err := NewREPL(Options{
-		ConfigPath:  configPath,
-		HistoryPath: historyPath,
-		Logger:      zap.NewNop(),
-	})
-	require.NoError(t, err)
-	defer repl.Close()
-
-	// Manually set contextProvider to nil to test edge case
-	repl.contextProvider = nil
-
-	// updatePredictorContext should not panic with nil contextProvider
-	repl.updatePredictorContext()
-}
-
-func TestREPL_ContextProviderWithoutHistory(t *testing.T) {
-	tmpDir := t.TempDir()
-	// Use an invalid history path that will cause history initialization to fail
-	historyPath := filepath.Join(tmpDir, "nonexistent_dir", "subdir", "history.db")
-	configPath := filepath.Join(tmpDir, "nonexistent.repl.gsh")
-
-	logger := zaptest.NewLogger(t)
-
-	repl, err := NewREPL(Options{
-		ConfigPath:  configPath,
-		HistoryPath: historyPath,
-		Logger:      logger,
-	})
-	require.NoError(t, err)
-	defer repl.Close()
-
-	// Context provider should still be initialized
-	assert.NotNil(t, repl.contextProvider)
-
-	// Get context - should still have basic retrievers
-	contextMap := repl.contextProvider.GetContext()
-
-	// Should have working_directory
-	_, hasWorkingDir := contextMap["working_directory"]
-	assert.True(t, hasWorkingDir, "context should include working_directory")
-
-	// Should have system_info
-	_, hasSystemInfo := contextMap["system_info"]
-	assert.True(t, hasSystemInfo, "context should include system_info")
-}
-
-func TestREPL_ContextContainsWorkingDirectory(t *testing.T) {
-	tmpDir := t.TempDir()
-	historyPath := filepath.Join(tmpDir, "history.db")
-	configPath := filepath.Join(tmpDir, "nonexistent.repl.gsh")
-
-	repl, err := NewREPL(Options{
-		ConfigPath:  configPath,
-		HistoryPath: historyPath,
-		Logger:      zap.NewNop(),
-	})
-	require.NoError(t, err)
-	defer repl.Close()
-
-	contextMap := repl.contextProvider.GetContext()
-
-	// Verify working_directory contains actual path
-	workingDir := contextMap["working_directory"]
-	assert.Contains(t, workingDir, "<working_dir>")
-	assert.Contains(t, workingDir, "</working_dir>")
-}
-
-func TestREPL_ContextContainsSystemInfo(t *testing.T) {
-	tmpDir := t.TempDir()
-	historyPath := filepath.Join(tmpDir, "history.db")
-	configPath := filepath.Join(tmpDir, "nonexistent.repl.gsh")
-
-	repl, err := NewREPL(Options{
-		ConfigPath:  configPath,
-		HistoryPath: historyPath,
-		Logger:      zap.NewNop(),
-	})
-	require.NoError(t, err)
-	defer repl.Close()
-
-	contextMap := repl.contextProvider.GetContext()
-
-	// Verify system_info contains OS and arch
-	systemInfo := contextMap["system_info"]
-	assert.Contains(t, systemInfo, "<system_info>")
-	assert.Contains(t, systemInfo, "OS:")
-	assert.Contains(t, systemInfo, "Arch:")
 }
 
 func TestREPL_HistoryPredictionWithoutLLM(t *testing.T) {
@@ -589,16 +470,15 @@ func TestREPL_HistoryPredictionWithoutLLM(t *testing.T) {
 	require.NoError(t, err)
 	require.GreaterOrEqual(t, len(entries), 2, "should have at least 2 history entries with 'echo' prefix")
 
-	// Create a history provider from the history manager
-	historyProvider := input.NewHistoryPredictionAdapter(repl.history)
-	require.NotNil(t, historyProvider)
+	// Create a mock provider that returns a prediction for instant trigger
+	provider := &mockPredictionProvider{
+		prediction: "echo hello world",
+	}
 
-	// Create prediction state with history but WITHOUT LLM provider
-	// This is the key test - it should not panic when llmProvider is nil
+	// Create prediction state with provider
 	predictionState := input.NewPredictionState(input.PredictionStateConfig{
-		HistoryProvider: historyProvider,
-		LLMProvider:     nil, // Explicitly nil - no LLM configured
-		Logger:          logger,
+		Provider: provider,
+		Logger:   logger,
 	})
 	require.NotNil(t, predictionState)
 
@@ -609,9 +489,8 @@ func TestREPL_HistoryPredictionWithoutLLM(t *testing.T) {
 	// Wait for the prediction result (with timeout)
 	select {
 	case result := <-resultCh:
-		// Should get a history-based prediction without error or panic
+		// Should get an instant prediction without error or panic
 		assert.NoError(t, result.Error, "prediction should not return an error")
-		assert.Equal(t, input.PredictionSourceHistory, result.Source, "prediction should come from history")
 		assert.Contains(t, result.Prediction, "echo", "prediction should start with the input prefix")
 	case <-time.After(1 * time.Second):
 		t.Fatal("prediction timed out")
@@ -672,14 +551,11 @@ tool testMiddleware(ctx, next) {
 	vars := interp.GetVariables()
 	toolVal, ok := vars["testMiddleware"]
 	require.True(t, ok)
-	tool, ok := toolVal.(*interpreter.ToolValue)
+	_, ok = toolVal.(*interpreter.ToolValue)
 	require.True(t, ok)
 
-	// Set up REPL context with middleware manager
-	replCtx := interp.SDKConfig().GetREPLContext()
-	require.NotNil(t, replCtx)
-	replCtx.MiddlewareManager = interpreter.NewMiddlewareManager()
-	replCtx.MiddlewareManager.Use(tool, interp)
+	// Register middleware using gsh.use("command.input", ...)
+	interp.EvalString(`gsh.use("command.input", testMiddleware)`, nil)
 
 	ctx := context.Background()
 
@@ -726,14 +602,11 @@ tool passThroughMiddleware(ctx, next) {
 	vars := interp.GetVariables()
 	toolVal, ok := vars["passThroughMiddleware"]
 	require.True(t, ok)
-	tool, ok := toolVal.(*interpreter.ToolValue)
+	_, ok = toolVal.(*interpreter.ToolValue)
 	require.True(t, ok)
 
-	// Set up REPL context with middleware manager
-	replCtx := interp.SDKConfig().GetREPLContext()
-	require.NotNil(t, replCtx)
-	replCtx.MiddlewareManager = interpreter.NewMiddlewareManager()
-	replCtx.MiddlewareManager.Use(tool, interp)
+	// Register middleware using gsh.use("command.input", ...)
+	interp.EvalString(`gsh.use("command.input", passThroughMiddleware)`, nil)
 
 	ctx := context.Background()
 
@@ -774,14 +647,11 @@ tool transformMiddleware(ctx, next) {
 	vars := interp.GetVariables()
 	toolVal, ok := vars["transformMiddleware"]
 	require.True(t, ok)
-	tool, ok := toolVal.(*interpreter.ToolValue)
+	_, ok = toolVal.(*interpreter.ToolValue)
 	require.True(t, ok)
 
-	// Set up REPL context with middleware manager
-	replCtx := interp.SDKConfig().GetREPLContext()
-	require.NotNil(t, replCtx)
-	replCtx.MiddlewareManager = interpreter.NewMiddlewareManager()
-	replCtx.MiddlewareManager.Use(tool, interp)
+	// Register middleware using gsh.use("command.input", ...)
+	interp.EvalString(`gsh.use("command.input", transformMiddleware)`, nil)
 
 	ctx := context.Background()
 
@@ -824,18 +694,9 @@ tool secondMiddleware(ctx, next) {
 	_, err = interp.EvalString(code, nil)
 	require.NoError(t, err)
 
-	vars := interp.GetVariables()
-	firstVal, _ := vars["firstMiddleware"]
-	first, _ := firstVal.(*interpreter.ToolValue)
-	secondVal, _ := vars["secondMiddleware"]
-	second, _ := secondVal.(*interpreter.ToolValue)
-
-	// Set up REPL context with middleware manager
-	replCtx := interp.SDKConfig().GetREPLContext()
-	require.NotNil(t, replCtx)
-	replCtx.MiddlewareManager = interpreter.NewMiddlewareManager()
-	replCtx.MiddlewareManager.Use(first, interp)
-	replCtx.MiddlewareManager.Use(second, interp)
+	// Register middleware using gsh.use("command.input", ...)
+	interp.EvalString(`gsh.use("command.input", firstMiddleware)`, nil)
+	interp.EvalString(`gsh.use("command.input", secondMiddleware)`, nil)
 
 	ctx := context.Background()
 
@@ -844,7 +705,7 @@ tool secondMiddleware(ctx, next) {
 	assert.NoError(t, err)
 
 	// Verify order: first registered = first to run
-	vars = interp.GetVariables()
+	vars := interp.GetVariables()
 	orderVal, ok := vars["__middlewareOrder"]
 	require.True(t, ok)
 	orderStr, ok := orderVal.(*interpreter.StringValue)
@@ -876,15 +737,8 @@ tool passThroughMiddleware(ctx, next) {
 	_, err = interp.EvalString(code, nil)
 	require.NoError(t, err)
 
-	vars := interp.GetVariables()
-	toolVal, _ := vars["passThroughMiddleware"]
-	tool, _ := toolVal.(*interpreter.ToolValue)
-
-	// Set up REPL context with middleware manager
-	replCtx := interp.SDKConfig().GetREPLContext()
-	require.NotNil(t, replCtx)
-	replCtx.MiddlewareManager = interpreter.NewMiddlewareManager()
-	replCtx.MiddlewareManager.Use(tool, interp)
+	// Register middleware using gsh.use("command.input", ...)
+	interp.EvalString(`gsh.use("command.input", passThroughMiddleware)`, nil)
 
 	ctx := context.Background()
 
@@ -892,3 +746,222 @@ tool passThroughMiddleware(ctx, next) {
 	err = repl.processCommand(ctx, "exit")
 	assert.Equal(t, ErrExit, err)
 }
+
+func TestREPL_ProcessCommand_SetsInterpreterContext(t *testing.T) {
+	tmpDir := t.TempDir()
+	historyPath := filepath.Join(tmpDir, "history.db")
+	configPath := filepath.Join(tmpDir, "nonexistent.repl.gsh")
+
+	repl, err := NewREPL(Options{
+		ConfigPath:  configPath,
+		HistoryPath: historyPath,
+		Logger:      zap.NewNop(),
+	})
+	require.NoError(t, err)
+	defer repl.Close()
+
+	interp := repl.executor.Interpreter()
+
+	// Track if context was set during middleware execution
+	var contextWasSet bool
+	var contextWasCancellable bool
+
+	// Create middleware that checks if context is set on interpreter
+	code := `
+tool checkContextMiddleware(ctx, next) {
+	return next(ctx)
+}
+`
+	_, err = interp.EvalString(code, nil)
+	require.NoError(t, err)
+
+	// We can't directly check from gsh script, so we'll use a Go-level check
+	// by wrapping the middleware execution
+	originalContext := interp.Context()
+
+	// Register middleware
+	interp.EvalString(`gsh.use("command.input", checkContextMiddleware)`, nil)
+
+	ctx := context.Background()
+
+	// Execute a simple command
+	err = repl.processCommand(ctx, "echo test")
+	assert.NoError(t, err)
+
+	// After command completes, context should be cleared (nil -> returns Background)
+	afterContext := interp.Context()
+
+	// The context before should be Background (default)
+	// This test verifies the basic flow works
+	assert.NotNil(t, originalContext, "original context should not be nil")
+	assert.NotNil(t, afterContext, "context after command should not be nil")
+
+	// Context should be reset after command (SetContext(nil) called)
+	// Both should effectively be background contexts
+	select {
+	case <-afterContext.Done():
+		t.Error("context after command should not be cancelled")
+	default:
+		// Expected - context is not cancelled
+	}
+
+	_ = contextWasSet
+	_ = contextWasCancellable
+}
+
+func TestREPL_ProcessCommand_ContextCancellationStopsShellCommand(t *testing.T) {
+	tmpDir := t.TempDir()
+	historyPath := filepath.Join(tmpDir, "history.db")
+	configPath := filepath.Join(tmpDir, "nonexistent.repl.gsh")
+
+	repl, err := NewREPL(Options{
+		ConfigPath:  configPath,
+		HistoryPath: historyPath,
+		Logger:      zap.NewNop(),
+	})
+	require.NoError(t, err)
+	defer repl.Close()
+
+	// Create a context that we'll cancel during command execution
+	ctx, cancel := context.WithCancel(context.Background())
+
+	// Start a long-running command in a goroutine
+	done := make(chan error, 1)
+	go func() {
+		// sleep 10 should be interrupted
+		done <- repl.processCommand(ctx, "/bin/sleep 10")
+	}()
+
+	// Give the command a moment to start
+	time.Sleep(100 * time.Millisecond)
+
+	// Cancel the context (simulates Ctrl+C effect)
+	cancel()
+
+	// Wait for command to finish (should be quick due to cancellation)
+	select {
+	case err := <-done:
+		// Command should complete (possibly with error due to cancellation)
+		// The important thing is it didn't run for 10 seconds
+		_ = err // Error may or may not be nil depending on timing
+	case <-time.After(2 * time.Second):
+		t.Fatal("command did not respond to context cancellation within timeout")
+	}
+}
+
+func TestREPL_ProcessCommand_InterruptRecordsExitCode130(t *testing.T) {
+	tmpDir := t.TempDir()
+	historyPath := filepath.Join(tmpDir, "history.db")
+	configPath := filepath.Join(tmpDir, "nonexistent.repl.gsh")
+
+	repl, err := NewREPL(Options{
+		ConfigPath:  configPath,
+		HistoryPath: historyPath,
+		Logger:      zap.NewNop(),
+	})
+	require.NoError(t, err)
+	defer repl.Close()
+
+	interp := repl.executor.Interpreter()
+
+	// Create middleware that simulates a long-running operation that gets interrupted
+	// We'll use a middleware that checks for context cancellation
+	code := `
+tool slowMiddleware(ctx, next) {
+	return { handled: true }
+}
+`
+	_, err = interp.EvalString(code, nil)
+	require.NoError(t, err)
+	interp.EvalString(`gsh.use("command.input", slowMiddleware)`, nil)
+
+	// For this test, we need to verify that when the signal handler detects
+	// SIGINT, it records exit code 130. Since we can't easily send SIGINT
+	// in a unit test, we verify the history recording works normally first.
+	ctx := context.Background()
+	err = repl.processCommand(ctx, "test command")
+	assert.NoError(t, err)
+
+	// Verify history was recorded
+	entries, err := repl.History().GetRecentEntries("", 10)
+	require.NoError(t, err)
+	require.Len(t, entries, 1)
+	assert.Equal(t, "test command", entries[0].Command)
+	// Exit code 0 for middleware-handled command
+	assert.True(t, entries[0].ExitCode.Valid)
+	assert.Equal(t, int32(0), entries[0].ExitCode.Int32)
+}
+
+func TestREPL_ProcessCommand_IgnoresSigintWhileWaitingForChild(t *testing.T) {
+	tmpDir := t.TempDir()
+	historyPath := filepath.Join(tmpDir, "history.db")
+	configPath := filepath.Join(tmpDir, "nonexistent.repl.gsh")
+
+	repl, err := NewREPL(Options{
+		ConfigPath:  configPath,
+		HistoryPath: historyPath,
+		Logger:      zap.NewNop(),
+	})
+	require.NoError(t, err)
+	defer repl.Close()
+
+	sigChan := make(chan os.Signal, 1)
+	repl.setSigintChannelFactory(func() (chan os.Signal, func()) {
+		return sigChan, func() {}
+	})
+
+	ctx := context.Background()
+	done := make(chan struct{})
+	go func() {
+		_ = repl.processCommand(ctx, "/bin/sleep 2")
+		close(done)
+	}()
+
+	// Give the command time to start and become the foreground job.
+	time.Sleep(200 * time.Millisecond)
+
+	// Deliver SIGINT to the REPL handler channel; the child should own it.
+	sigChan <- syscall.SIGINT
+
+	// The command should keep running for a bit instead of being interrupted immediately.
+	select {
+	case <-done:
+		t.Fatal("command was interrupted by parent SIGINT")
+	case <-time.After(200 * time.Millisecond):
+	}
+
+	<-done
+
+	entries, err := repl.History().GetRecentEntries("", 10)
+	require.NoError(t, err)
+	require.Len(t, entries, 1)
+	assert.True(t, entries[0].ExitCode.Valid)
+	assert.Equal(t, int32(0), entries[0].ExitCode.Int32)
+}
+
+func TestREPL_ProcessCommand_ContextPassedToShellExecution(t *testing.T) {
+	tmpDir := t.TempDir()
+	historyPath := filepath.Join(tmpDir, "history.db")
+	configPath := filepath.Join(tmpDir, "nonexistent.repl.gsh")
+
+	repl, err := NewREPL(Options{
+		ConfigPath:  configPath,
+		HistoryPath: historyPath,
+		Logger:      zap.NewNop(),
+	})
+	require.NoError(t, err)
+	defer repl.Close()
+
+	// Create a pre-cancelled context
+	ctx, cancel := context.WithCancel(context.Background())
+	cancel() // Cancel immediately
+
+	// Try to run a command with already-cancelled context
+	// The command should fail quickly or return an error
+	start := time.Now()
+	_ = repl.processCommand(ctx, "/bin/sleep 5")
+	elapsed := time.Since(start)
+
+	// Should complete much faster than 5 seconds because context is cancelled
+	assert.Less(t, elapsed, 2*time.Second, "cancelled context should prevent long-running command")
+}
