@@ -57,6 +57,16 @@ func (i *Interpreter) registerGshSDK() {
 	// Create gsh.ui object for UI control (spinner, styles, cursor)
 	uiObj := i.createUIObject()
 
+	// Create gsh.history object for command history access
+	historyObj := i.createHistoryObject()
+
+	// Create gsh.currentDirectory (dynamic, reads from interpreter's working directory)
+	currentDirectoryObj := &DynamicValue{
+		Get: func() Value {
+			return &StringValue{Value: i.GetWorkingDir()}
+		},
+	}
+
 	// Create Math object with common methods and constants
 	mathObj := &ObjectValue{
 		Properties: map[string]*PropertyDescriptor{
@@ -143,22 +153,20 @@ func (i *Interpreter) registerGshSDK() {
 			"ui":               {Value: uiObj, ReadOnly: true},
 			"models":           {Value: modelsObj, ReadOnly: true},
 			"lastCommand":      {Value: lastCommandObj, ReadOnly: true},
+			"history":          {Value: historyObj, ReadOnly: true},
+			"currentDirectory": {Value: currentDirectoryObj, ReadOnly: true},
 			"prompt":           {Value: promptObj},
-			"on": {Value: &BuiltinValue{
-				Name: "gsh.on",
-				Fn:   i.builtinGshOn,
+			"use": {Value: &BuiltinValue{
+				Name: "gsh.use",
+				Fn:   i.builtinGshUse,
 			}, ReadOnly: true},
-			"off": {Value: &BuiltinValue{
-				Name: "gsh.off",
-				Fn:   i.builtinGshOff,
+			"remove": {Value: &BuiltinValue{
+				Name: "gsh.remove",
+				Fn:   i.builtinGshRemove,
 			}, ReadOnly: true},
-			"useCommandMiddleware": {Value: &BuiltinValue{
-				Name: "gsh.useCommandMiddleware",
-				Fn:   i.builtinGshUseCommandMiddleware,
-			}, ReadOnly: true},
-			"removeCommandMiddleware": {Value: &BuiltinValue{
-				Name: "gsh.removeCommandMiddleware",
-				Fn:   i.builtinGshRemoveCommandMiddleware,
+			"removeAll": {Value: &BuiltinValue{
+				Name: "gsh.removeAll",
+				Fn:   i.builtinGshRemoveAll,
 			}, ReadOnly: true},
 		},
 	}
@@ -170,54 +178,76 @@ func (i *Interpreter) registerGshSDK() {
 	dateTimeObj := createDateTimeObject()
 	i.env.Set("DateTime", dateTimeObj)
 
+	// Register Regexp as a global object (not under gsh)
+	regexpObj := createRegexpObject()
+	i.env.Set("Regexp", regexpObj)
+
 	i.env.Set("gsh", gshObj)
 }
 
-// builtinGshOn implements gsh.on(event, handler)
-func (i *Interpreter) builtinGshOn(args []Value) (Value, error) {
+// builtinGshUse implements gsh.use(event, handler)
+// Registers a middleware handler for an event. Returns a unique handler ID.
+// Middleware handlers have the signature: tool(ctx, next)
+func (i *Interpreter) builtinGshUse(args []Value) (Value, error) {
 	if len(args) != 2 {
-		return nil, fmt.Errorf("gsh.on() takes 2 arguments (event: string, handler: tool), got %d", len(args))
+		return nil, fmt.Errorf("gsh.use() takes 2 arguments (event: string, handler: tool), got %d", len(args))
 	}
 
 	eventName, ok := args[0].(*StringValue)
 	if !ok {
-		return nil, fmt.Errorf("gsh.on() first argument must be a string (event name), got %s", args[0].Type())
+		return nil, fmt.Errorf("gsh.use() first argument must be a string (event name), got %s", args[0].Type())
 	}
 
 	handler, ok := args[1].(*ToolValue)
 	if !ok {
-		return nil, fmt.Errorf("gsh.on() second argument must be a tool, got %s", args[1].Type())
+		return nil, fmt.Errorf("gsh.use() second argument must be a tool, got %s", args[1].Type())
+	}
+
+	// Validate tool has correct signature (ctx, next)
+	if len(handler.Parameters) != 2 {
+		return nil, fmt.Errorf("middleware handler must take 2 parameters (ctx, next), got %d", len(handler.Parameters))
 	}
 
 	// Register the handler and return the handler ID
-	handlerID := i.eventManager.On(eventName.Value, handler)
+	handlerID := i.eventManager.Use(eventName.Value, handler)
 	return &StringValue{Value: handlerID}, nil
 }
 
-// builtinGshOff implements gsh.off(event, handlerID?)
-func (i *Interpreter) builtinGshOff(args []Value) (Value, error) {
-	if len(args) < 1 || len(args) > 2 {
-		return nil, fmt.Errorf("gsh.off() takes 1 or 2 arguments (event: string, handlerID?: string), got %d", len(args))
+// builtinGshRemove implements gsh.remove(event, handler)
+// Removes a previously registered middleware handler by tool reference.
+func (i *Interpreter) builtinGshRemove(args []Value) (Value, error) {
+	if len(args) != 2 {
+		return nil, fmt.Errorf("gsh.remove() takes 2 arguments (event: string, handler: tool), got %d", len(args))
 	}
 
 	eventName, ok := args[0].(*StringValue)
 	if !ok {
-		return nil, fmt.Errorf("gsh.off() first argument must be a string (event name), got %s", args[0].Type())
+		return nil, fmt.Errorf("gsh.remove() first argument must be a string (event name), got %s", args[0].Type())
 	}
 
-	handlerID := ""
-	if len(args) == 2 {
-		if args[1].Type() != ValueTypeNull {
-			handlerIDVal, ok := args[1].(*StringValue)
-			if !ok {
-				return nil, fmt.Errorf("gsh.off() second argument must be a string (handler ID) or null, got %s", args[1].Type())
-			}
-			handlerID = handlerIDVal.Value
-		}
+	handler, ok := args[1].(*ToolValue)
+	if !ok {
+		return nil, fmt.Errorf("gsh.remove() second argument must be a tool, got %s", args[1].Type())
 	}
 
-	i.eventManager.Off(eventName.Value, handlerID)
-	return &NullValue{}, nil
+	removed := i.eventManager.Remove(eventName.Value, handler)
+	return &BoolValue{Value: removed}, nil
+}
+
+// builtinGshRemoveAll implements gsh.removeAll(event)
+// Removes all registered handlers for an event. Returns the number of handlers removed.
+func (i *Interpreter) builtinGshRemoveAll(args []Value) (Value, error) {
+	if len(args) != 1 {
+		return nil, fmt.Errorf("gsh.removeAll() takes 1 argument (event: string), got %d", len(args))
+	}
+
+	eventName, ok := args[0].(*StringValue)
+	if !ok {
+		return nil, fmt.Errorf("gsh.removeAll() argument must be a string (event name), got %s", args[0].Type())
+	}
+
+	count := i.eventManager.RemoveAll(eventName.Value)
+	return &NumberValue{Value: float64(count)}, nil
 }
 
 // builtinMathRandom implements Math.random()
@@ -627,61 +657,6 @@ func (g *GshObjectValue) SetProperty(name string, value Value) error {
 	}
 }
 
-// builtinGshUseCommandMiddleware implements gsh.useCommandMiddleware(middleware)
-// Registers a command middleware function. Returns a unique ID that can be used to remove it.
-// Command middleware runs when the user submits input (presses Enter).
-func (i *Interpreter) builtinGshUseCommandMiddleware(args []Value) (Value, error) {
-	if len(args) != 1 {
-		return nil, fmt.Errorf("gsh.useCommandMiddleware() takes 1 argument (middleware: tool), got %d", len(args))
-	}
-
-	tool, ok := args[0].(*ToolValue)
-	if !ok {
-		return nil, fmt.Errorf("gsh.useCommandMiddleware() argument must be a tool, got %s", args[0].Type())
-	}
-
-	// Validate tool has correct signature (ctx, next)
-	if len(tool.Parameters) != 2 {
-		return nil, fmt.Errorf("middleware tool must take 2 parameters (ctx, next), got %d", len(tool.Parameters))
-	}
-
-	replCtx := i.sdkConfig.GetREPLContext()
-	if replCtx == nil {
-		return nil, fmt.Errorf("gsh.useCommandMiddleware() is only available in the REPL")
-	}
-
-	// Initialize middleware manager if needed
-	if replCtx.MiddlewareManager == nil {
-		replCtx.MiddlewareManager = NewMiddlewareManager()
-	}
-
-	// Register the middleware
-	id := replCtx.MiddlewareManager.Use(tool, replCtx.Interpreter)
-	return &StringValue{Value: id}, nil
-}
-
-// builtinGshRemoveCommandMiddleware implements gsh.removeCommandMiddleware(middleware)
-// Removes a previously registered command middleware by tool reference.
-func (i *Interpreter) builtinGshRemoveCommandMiddleware(args []Value) (Value, error) {
-	if len(args) != 1 {
-		return nil, fmt.Errorf("gsh.removeCommandMiddleware() takes 1 argument (middleware: tool), got %d", len(args))
-	}
-
-	tool, ok := args[0].(*ToolValue)
-	if !ok {
-		return nil, fmt.Errorf("gsh.removeCommandMiddleware() argument must be a tool, got %s", args[0].Type())
-	}
-
-	replCtx := i.sdkConfig.GetREPLContext()
-	if replCtx == nil || replCtx.MiddlewareManager == nil {
-		return &BoolValue{Value: false}, nil
-	}
-
-	// Remove by tool reference
-	removed := replCtx.MiddlewareManager.RemoveByTool(tool)
-	return &BoolValue{Value: removed}, nil
-}
-
 // ModelsObjectValue represents the gsh.models object.
 // It holds pre-created SDKModelRef instances for each tier to avoid allocations
 // on repeated access and to maintain pointer identity.
@@ -780,6 +755,8 @@ func (c *LastCommandObjectValue) GetProperty(name string) Value {
 		return &NullValue{}
 	}
 	switch name {
+	case "command":
+		return &StringValue{Value: c.lastCommand.Command}
 	case "exitCode":
 		return &NumberValue{Value: float64(c.lastCommand.ExitCode)}
 	case "durationMs":
@@ -806,3 +783,130 @@ func (i *Interpreter) createNativeToolsObject() *ObjectValue {
 		},
 	}
 }
+
+// createHistoryObject creates the gsh.history object for command history access.
+func (i *Interpreter) createHistoryObject() *ObjectValue {
+	return &ObjectValue{
+		Properties: map[string]*PropertyDescriptor{
+			"findPrefix": {Value: &BuiltinValue{
+				Name: "gsh.history.findPrefix",
+				Fn:   i.builtinHistoryFindPrefix,
+			}, ReadOnly: true},
+			"getRecent": {Value: &BuiltinValue{
+				Name: "gsh.history.getRecent",
+				Fn:   i.builtinHistoryGetRecent,
+			}, ReadOnly: true},
+		},
+	}
+}
+
+// builtinHistoryFindPrefix implements gsh.history.findPrefix(prefix, limit)
+// Returns an array of history entries that start with the given prefix, ordered by most recent first.
+// Each entry is an object with { command, exitCode, timestamp }.
+// Parameters:
+//   - prefix (string): The prefix to search for
+//   - limit (number, optional): Maximum number of history entries to return (default: 10)
+func (i *Interpreter) builtinHistoryFindPrefix(args []Value) (Value, error) {
+	if len(args) < 1 || len(args) > 2 {
+		return nil, fmt.Errorf("gsh.history.findPrefix() takes 1-2 arguments (prefix: string, limit?: number), got %d", len(args))
+	}
+
+	// Get prefix argument
+	prefix, ok := args[0].(*StringValue)
+	if !ok {
+		return nil, fmt.Errorf("gsh.history.findPrefix() first argument must be a string, got %s", args[0].Type())
+	}
+
+	// Get optional limit argument (default: 10)
+	limit := 10
+	if len(args) == 2 {
+		limitVal, ok := args[1].(*NumberValue)
+		if !ok {
+			return nil, fmt.Errorf("gsh.history.findPrefix() second argument must be a number, got %s", args[1].Type())
+		}
+		limit = int(limitVal.Value)
+		if limit <= 0 {
+			limit = 10
+		}
+	}
+
+	// Get history provider from SDK config
+	provider := i.sdkConfig.GetHistoryProvider()
+	if provider == nil {
+		// No history provider available (e.g., in script mode)
+		return &ArrayValue{Elements: []Value{}}, nil
+	}
+
+	// Search history for matching prefix
+	entries, err := provider.FindPrefix(prefix.Value, limit)
+	if err != nil {
+		return &ArrayValue{Elements: []Value{}}, nil // Return empty array on error
+	}
+
+	// Convert to array of objects
+	elements := make([]Value, len(entries))
+	for i, entry := range entries {
+		elements[i] = &ObjectValue{
+			Properties: map[string]*PropertyDescriptor{
+				"command":   {Value: &StringValue{Value: entry.Command}},
+				"exitCode":  {Value: &NumberValue{Value: float64(entry.ExitCode)}},
+				"timestamp": {Value: &NumberValue{Value: float64(entry.Timestamp)}},
+			},
+		}
+	}
+
+	return &ArrayValue{Elements: elements}, nil
+}
+
+// builtinHistoryGetRecent implements gsh.history.getRecent(limit)
+// Returns an array of the most recent history entries in chronological order
+// (oldest first, most recent last). This ordering is ideal for providing context
+// to LLMs as it shows the natural flow of commands.
+// Each entry is an object with { command, exitCode, timestamp }.
+// Parameters:
+//   - limit (number, optional): Maximum number of history entries to return (default: 10)
+func (i *Interpreter) builtinHistoryGetRecent(args []Value) (Value, error) {
+	if len(args) > 1 {
+		return nil, fmt.Errorf("gsh.history.getRecent() takes 0-1 arguments (limit?: number), got %d", len(args))
+	}
+
+	// Get optional limit argument (default: 10)
+	limit := 10
+	if len(args) == 1 {
+		limitVal, ok := args[0].(*NumberValue)
+		if !ok {
+			return nil, fmt.Errorf("gsh.history.getRecent() argument must be a number, got %s", args[0].Type())
+		}
+		limit = int(limitVal.Value)
+		if limit <= 0 {
+			limit = 10
+		}
+	}
+
+	// Get history provider from SDK config
+	provider := i.sdkConfig.GetHistoryProvider()
+	if provider == nil {
+		// No history provider available (e.g., in script mode)
+		return &ArrayValue{Elements: []Value{}}, nil
+	}
+
+	// Get recent history entries
+	entries, err := provider.GetRecent(limit)
+	if err != nil {
+		return &ArrayValue{Elements: []Value{}}, nil // Return empty array on error
+	}
+
+	// Convert to array of objects
+	elements := make([]Value, len(entries))
+	for i, entry := range entries {
+		elements[i] = &ObjectValue{
+			Properties: map[string]*PropertyDescriptor{
+				"command":   {Value: &StringValue{Value: entry.Command}},
+				"exitCode":  {Value: &NumberValue{Value: float64(entry.ExitCode)}},
+				"timestamp": {Value: &NumberValue{Value: float64(entry.Timestamp)}},
+			},
+		}
+	}
+
+	return &ArrayValue{Elements: elements}, nil
+}
