@@ -3,6 +3,7 @@ package interpreter
 import (
 	"bufio"
 	"bytes"
+	"context"
 	"encoding/json"
 	"fmt"
 	"io"
@@ -77,8 +78,9 @@ func convertMessageContent(msg ChatMessage) []openAIContentPart {
 	}
 }
 
-// ChatCompletion sends a chat completion request to OpenAI
-func (p *OpenAIProvider) ChatCompletion(request ChatRequest) (*ChatResponse, error) {
+// ChatCompletion sends a chat completion request to OpenAI.
+// The ctx parameter allows cancellation of the request (e.g., via Ctrl+C).
+func (p *OpenAIProvider) ChatCompletion(ctx context.Context, request ChatRequest) (*ChatResponse, error) {
 	if request.Model == nil {
 		return nil, fmt.Errorf("OpenAI provider requires a model")
 	}
@@ -199,14 +201,22 @@ func (p *OpenAIProvider) ChatCompletion(request ChatRequest) (*ChatResponse, err
 		}
 	}
 
-	// Marshal request
-	reqBody, err := json.Marshal(openaiReq)
+	// Get extraBody if configured
+	var extraBody map[string]interface{}
+	if extraBodyVal, ok := request.Model.Config["extraBody"]; ok {
+		if extraBodyObj, ok := extraBodyVal.(*ObjectValue); ok {
+			extraBody = objectValueToMap(extraBodyObj)
+		}
+	}
+
+	// Marshal request with extraBody merged at top level
+	reqBody, err := marshalRequestWithExtraBody(openaiReq, extraBody)
 	if err != nil {
 		return nil, fmt.Errorf("failed to marshal request: %w", err)
 	}
 
-	// Create HTTP request
-	httpReq, err := http.NewRequest("POST", apiURL, bytes.NewBuffer(reqBody))
+	// Create HTTP request with context for cancellation support
+	httpReq, err := http.NewRequestWithContext(ctx, "POST", apiURL, bytes.NewBuffer(reqBody))
 	if err != nil {
 		return nil, fmt.Errorf("failed to create request: %w", err)
 	}
@@ -296,8 +306,9 @@ func (p *OpenAIProvider) ChatCompletion(request ChatRequest) (*ChatResponse, err
 }
 
 // StreamingChatCompletion sends a chat completion request with streaming response.
+// The ctx parameter allows cancellation of the streaming request (e.g., via Ctrl+C).
 // The callbacks provide hooks for content chunks and tool call detection.
-func (p *OpenAIProvider) StreamingChatCompletion(request ChatRequest, callbacks *StreamCallbacks) (*ChatResponse, error) {
+func (p *OpenAIProvider) StreamingChatCompletion(ctx context.Context, request ChatRequest, callbacks *StreamCallbacks) (*ChatResponse, error) {
 	if request.Model == nil {
 		return nil, fmt.Errorf("OpenAI provider requires a model")
 	}
@@ -422,14 +433,22 @@ func (p *OpenAIProvider) StreamingChatCompletion(request ChatRequest, callbacks
 		}
 	}
 
-	// Marshal request
-	reqBody, err := json.Marshal(openaiReq)
+	// Get extraBody if configured
+	var extraBody map[string]interface{}
+	if extraBodyVal, ok := request.Model.Config["extraBody"]; ok {
+		if extraBodyObj, ok := extraBodyVal.(*ObjectValue); ok {
+			extraBody = objectValueToMap(extraBodyObj)
+		}
+	}
+
+	// Marshal request with extraBody merged at top level
+	reqBody, err := marshalRequestWithExtraBody(openaiReq, extraBody)
 	if err != nil {
 		return nil, fmt.Errorf("failed to marshal request: %w", err)
 	}
 
-	// Create HTTP request
-	httpReq, err := http.NewRequest("POST", apiURL, bytes.NewBuffer(reqBody))
+	// Create HTTP request with context for cancellation support
+	httpReq, err := http.NewRequestWithContext(ctx, "POST", apiURL, bytes.NewBuffer(reqBody))
 	if err != nil {
 		return nil, fmt.Errorf("failed to create request: %w", err)
 	}
@@ -474,6 +493,11 @@ func (p *OpenAIProvider) StreamingChatCompletion(request ChatRequest, callbacks
 
 	scanner := bufio.NewScanner(resp.Body)
 	for scanner.Scan() {
+		// Check for cancellation before processing each chunk
+		if callbacks != nil && callbacks.ShouldCancel != nil && callbacks.ShouldCancel() {
+			return nil, fmt.Errorf("streaming cancelled")
+		}
+
 		line := scanner.Text()
 
 		// Skip empty lines
@@ -596,6 +620,45 @@ func (p *OpenAIProvider) StreamingChatCompletion(request ChatRequest, callbacks
 	return response, nil
 }
 
+// objectValueToMap converts an ObjectValue to a map[string]interface{} for JSON serialization.
+func objectValueToMap(obj *ObjectValue) map[string]interface{} {
+	result := make(map[string]interface{})
+	for key := range obj.Properties {
+		val := obj.GetPropertyValue(key)
+		result[key] = ValueToInterface(val)
+	}
+	return result
+}
+
+// marshalRequestWithExtraBody marshals a request struct to JSON and merges extraBody
+// properties as top-level fields in the resulting JSON object.
+func marshalRequestWithExtraBody(req interface{}, extraBody map[string]interface{}) ([]byte, error) {
+	// First marshal the struct to JSON
+	baseJSON, err := json.Marshal(req)
+	if err != nil {
+		return nil, err
+	}
+
+	// If no extraBody, return the base JSON as-is
+	if len(extraBody) == 0 {
+		return baseJSON, nil
+	}
+
+	// Unmarshal into a map so we can add extra fields
+	var merged map[string]interface{}
+	if err := json.Unmarshal(baseJSON, &merged); err != nil {
+		return nil, err
+	}
+
+	// Merge extraBody properties at top level
+	for key, value := range extraBody {
+		merged[key] = value
+	}
+
+	// Marshal the merged result
+	return json.Marshal(merged)
+}
+
 // OpenAI-specific types
 
 type openAIStreamingChatCompletionRequest struct {
