@@ -1,6 +1,7 @@
 package interpreter
 
 import (
+	"context"
 	"fmt"
 	"strconv"
 	"strings"
@@ -36,6 +37,10 @@ const (
 	ValueTypeMap
 	// ValueTypeSet represents a set value
 	ValueTypeSet
+	// ValueTypeACP represents an ACP (Agent Client Protocol) agent configuration
+	ValueTypeACP
+	// ValueTypeACPSession represents an active ACP session
+	ValueTypeACPSession
 )
 
 // String returns the string representation of the value type
@@ -67,6 +72,10 @@ func (vt ValueType) String() string {
 		return "map"
 	case ValueTypeSet:
 		return "set"
+	case ValueTypeACP:
+		return "acp"
+	case ValueTypeACPSession:
+		return "acpsession"
 	default:
 		return "unknown"
 	}
@@ -499,24 +508,26 @@ func (m *ModelValue) GetModel() *ModelValue {
 
 // ChatCompletion performs a chat completion using this model's provider.
 // This is a convenience method that delegates to the model's provider.
-func (m *ModelValue) ChatCompletion(request ChatRequest) (*ChatResponse, error) {
+// The ctx parameter allows cancellation of the request (e.g., via Ctrl+C).
+func (m *ModelValue) ChatCompletion(ctx context.Context, request ChatRequest) (*ChatResponse, error) {
 	if m.Provider == nil {
 		return nil, fmt.Errorf("model '%s' has no provider configured", m.Name)
 	}
 	// Ensure the request uses this model
 	request.Model = m
-	return m.Provider.ChatCompletion(request)
+	return m.Provider.ChatCompletion(ctx, request)
 }
 
 // StreamingChatCompletion performs a streaming chat completion using this model's provider.
 // This is a convenience method that delegates to the model's provider.
-func (m *ModelValue) StreamingChatCompletion(request ChatRequest, callbacks *StreamCallbacks) (*ChatResponse, error) {
+// The ctx parameter allows cancellation of the streaming request (e.g., via Ctrl+C).
+func (m *ModelValue) StreamingChatCompletion(ctx context.Context, request ChatRequest, callbacks *StreamCallbacks) (*ChatResponse, error) {
 	if m.Provider == nil {
 		return nil, fmt.Errorf("model '%s' has no provider configured", m.Name)
 	}
 	// Ensure the request uses this model
 	request.Model = m
-	return m.Provider.StreamingChatCompletion(request, callbacks)
+	return m.Provider.StreamingChatCompletion(ctx, request, callbacks)
 }
 
 // AgentValue represents an agent configuration
@@ -537,6 +548,22 @@ func (a *AgentValue) Equals(other Value) bool {
 	return false
 }
 
+// GetProperty returns a property of the agent.
+// Supports "name" for the agent name, "metadata" for user-defined metadata,
+// and any other config field.
+func (a *AgentValue) GetProperty(name string) Value {
+	switch name {
+	case "name":
+		return &StringValue{Value: a.Name}
+	default:
+		// Check if the property exists in the Config map
+		if val, ok := a.Config[name]; ok {
+			return val
+		}
+		return &NullValue{}
+	}
+}
+
 // ConversationValue represents a conversation state
 type ConversationValue struct {
 	// Messages in the conversation history
@@ -555,6 +582,57 @@ func (c *ConversationValue) Equals(other Value) bool {
 	return false
 }
 
+// GetProperty returns a property of the conversation
+func (c *ConversationValue) GetProperty(name string) Value {
+	switch name {
+	case "messages":
+		// Convert []ChatMessage to ArrayValue of ObjectValues
+		elements := make([]Value, len(c.Messages))
+		for i, msg := range c.Messages {
+			elements[i] = chatMessageToObjectValue(msg)
+		}
+		return &ArrayValue{Elements: elements}
+	case "lastMessage":
+		if len(c.Messages) == 0 {
+			return &NullValue{}
+		}
+		return chatMessageToObjectValue(c.Messages[len(c.Messages)-1])
+	default:
+		return &NullValue{}
+	}
+}
+
+// chatMessageToObjectValue converts a ChatMessage to an ObjectValue for script access
+func chatMessageToObjectValue(msg ChatMessage) *ObjectValue {
+	props := map[string]*PropertyDescriptor{
+		"role":    {Value: &StringValue{Value: msg.Role}},
+		"content": {Value: &StringValue{Value: msg.Content}},
+	}
+
+	// Include optional fields if present
+	if msg.Name != "" {
+		props["name"] = &PropertyDescriptor{Value: &StringValue{Value: msg.Name}}
+	}
+	if msg.ToolCallID != "" {
+		props["toolCallId"] = &PropertyDescriptor{Value: &StringValue{Value: msg.ToolCallID}}
+	}
+	if len(msg.ToolCalls) > 0 {
+		toolCalls := make([]Value, len(msg.ToolCalls))
+		for i, tc := range msg.ToolCalls {
+			toolCalls[i] = &ObjectValue{
+				Properties: map[string]*PropertyDescriptor{
+					"id":        {Value: &StringValue{Value: tc.ID}},
+					"name":      {Value: &StringValue{Value: tc.Name}},
+					"arguments": {Value: InterfaceToValue(tc.Arguments)},
+				},
+			}
+		}
+		props["toolCalls"] = &PropertyDescriptor{Value: &ArrayValue{Elements: toolCalls}}
+	}
+
+	return &ObjectValue{Properties: props}
+}
+
 // MapValue represents a map value (key-value pairs)
 type MapValue struct {
 	Entries map[string]Value
@@ -735,6 +813,106 @@ func (r *SDKModelRef) SetProperty(name string, value Value) error {
 	return model.SetProperty(name, value)
 }
 
+// ACPValue represents an ACP (Agent Client Protocol) agent configuration.
+// Unlike AgentValue which has a local model and tools, ACPValue connects
+// to an external agent process via the ACP protocol.
+type ACPValue struct {
+	Name   string           // Agent name from declaration
+	Config map[string]Value // Configuration (command, args, env, cwd, etc.)
+}
+
+func (a *ACPValue) Type() ValueType { return ValueTypeACP }
+func (a *ACPValue) String() string {
+	return fmt.Sprintf("<acp %s>", a.Name)
+}
+func (a *ACPValue) IsTruthy() bool { return true }
+func (a *ACPValue) Equals(other Value) bool {
+	if otherACP, ok := other.(*ACPValue); ok {
+		return a.Name == otherACP.Name
+	}
+	return false
+}
+
+// GetProperty returns a property of the ACP agent configuration
+func (a *ACPValue) GetProperty(name string) Value {
+	switch name {
+	case "name":
+		return &StringValue{Value: a.Name}
+	default:
+		// Check if the property exists in the Config map
+		if val, ok := a.Config[name]; ok {
+			return val
+		}
+		return &NullValue{}
+	}
+}
+
+// ACPSessionValue represents an active session with an ACP agent.
+// It tracks the conversation messages locally for property access,
+// though the authoritative state is maintained by the external agent.
+type ACPSessionValue struct {
+	// Agent is the ACP agent this session is bound to
+	Agent *ACPValue
+
+	// Messages tracks conversation history locally for property access.
+	// This is a local copy - the external agent owns the authoritative state.
+	Messages []ChatMessage
+
+	// SessionID is the external agent's session identifier (set after initialization)
+	SessionID string
+
+	// Closed indicates if the session has been terminated
+	Closed bool
+}
+
+func (s *ACPSessionValue) Type() ValueType { return ValueTypeACPSession }
+func (s *ACPSessionValue) String() string {
+	if s.Closed {
+		return fmt.Sprintf("<acpsession %s (closed) with %d messages>", s.Agent.Name, len(s.Messages))
+	}
+	return fmt.Sprintf("<acpsession %s with %d messages>", s.Agent.Name, len(s.Messages))
+}
+func (s *ACPSessionValue) IsTruthy() bool { return !s.Closed && len(s.Messages) > 0 }
+func (s *ACPSessionValue) Equals(other Value) bool {
+	if otherSession, ok := other.(*ACPSessionValue); ok {
+		return s.SessionID == otherSession.SessionID && s.Agent.Name == otherSession.Agent.Name
+	}
+	return false
+}
+
+// GetProperty returns a property of the ACP session.
+// Mirrors ConversationValue's property interface for consistency.
+func (s *ACPSessionValue) GetProperty(name string) Value {
+	switch name {
+	case "messages":
+		// Convert []ChatMessage to ArrayValue of ObjectValues
+		elements := make([]Value, len(s.Messages))
+		for i, msg := range s.Messages {
+			elements[i] = chatMessageToObjectValue(msg)
+		}
+		return &ArrayValue{Elements: elements}
+	case "lastMessage":
+		if len(s.Messages) == 0 {
+			return &NullValue{}
+		}
+		return chatMessageToObjectValue(s.Messages[len(s.Messages)-1])
+	case "agent":
+		return s.Agent
+	case "sessionId":
+		return &StringValue{Value: s.SessionID}
+	case "closed":
+		return &BoolValue{Value: s.Closed}
+	default:
+		return &NullValue{}
+	}
+}
+
+// Close marks the session as closed.
+// This is called when session.close() is invoked from gsh script.
+func (s *ACPSessionValue) Close() {
+	s.Closed = true
+}
+
 // UnwrapValue unwraps a DynamicValue to its underlying value.
 // If the value is not a DynamicValue (or doesn't implement DynamicValueGetter),
 // it returns the original value unchanged.
